import "reflect-metadata";
/**
 * A function to provide constructor parameter values
 * If a parameter was passed to the wrapped constructor it is passed as the first argument
 * Reflect Metadata (usually the type of the parameter) is passed as the second argument
 */
export declare type ParameterProvider = (passedParameter: any, reflectData: any) => any;
declare type Constructor = new (...args: any[]) => any;
declare type Function = (...args: any[]) => any;
declare type ConstructorParams<T extends Constructor> = T extends new (...args: infer P) => any ? P : never;
declare type FunctionParams<T extends Function> = T extends (...args: infer P) => any ? P : never;
declare type AnyParams<T extends any[]> = {
    [P in keyof T]: any;
};
declare type InjectedConstructor<T extends Constructor, P extends any[]> = T extends new (...args: any[]) => infer R ? new (...args: P) => R : never;
declare type InjectedFunction<T extends Function, P extends any[]> = T extends (...args: any) => infer R ? (...args: P) => R : never;
/**
 * Converts a class constructor with parameters into a constructor with optional parameters.
 * Where possible parameter values are provided when no value is passed to the new constructor.
 * This logic is determied by the parameterProvider function.
 *
 * @param type Type to be constructed
 * @param parameterProvider function to lookup the constructor parameter value
 */
export declare function injectConstructor<T extends new (...args: any[]) => any>(type: T, parameterProvider: ParameterProvider): InjectedConstructor<T, Partial<ConstructorParams<T>>>;
/**
 * Wraps a function and provides arguments from parameterProvider
 * Where possible argument values are provided when no value is passed to the new constructor.
 * This logic is determied by the parameterProvider function.
 *
 * @param type Type to be constructed
 * @param paramTypes The types to be constructed (unfortunately this can't be done by reflection)
 * @param parameterProvider function to lookup the constructor parameter value
 */
export declare function injectFunction<T extends Function>(func: T, paramTypes: AnyParams<FunctionParams<T>>, parameterProvider: ParameterProvider): InjectedFunction<T, Partial<FunctionParams<T>>>;
export {};
