"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
/**
 * Converts a class constructor with parameters into a constructor with optional parameters.
 * Where possible parameter values are provided when no value is passed to the new constructor.
 * This logic is determied by the parameterProvider function.
 *
 * @param type Type to be constructed
 * @param parameterProvider function to lookup the constructor parameter value
 */
function injectConstructor(type, parameterProvider) {
    return createClassOptionalConstructorParams(type, parameterProvider);
}
exports.injectConstructor = injectConstructor;
/**
 * Wraps a function and provides arguments from parameterProvider
 * Where possible argument values are provided when no value is passed to the new constructor.
 * This logic is determied by the parameterProvider function.
 *
 * @param type Type to be constructed
 * @param paramTypes The types to be constructed (unfortunately this can't be done by reflection)
 * @param parameterProvider function to lookup the constructor parameter value
 */
function injectFunction(func, paramTypes, parameterProvider) {
    return createFunctionOptionalParams(func, paramTypes, parameterProvider);
}
exports.injectFunction = injectFunction;
function createClassOptionalConstructorParams(type, parameterProvider) {
    return wrapClassConstructor(type, parameterProvider);
}
function createFunctionOptionalParams(func, paramTypes, parameterProvider) {
    return wrapFunction(func, paramTypes, parameterProvider);
}
function wrapClassConstructor(type, parameterProvider) {
    const params = Reflect.getMetadata("design:paramtypes", type);
    if (params == null || params.length == null || params.length === 0) {
        return type;
    }
    return class extends type {
        constructor(...args) {
            function resolveParameter(paramReflect, index) {
                return parameterProvider(args[index], paramReflect);
            }
            const constructorParams = params.map(resolveParameter);
            super(...constructorParams);
        }
    };
}
// Decorator just to ensure that we get metadata for classes
function saveMetaData() {
    // tslint:disable-next-line: no-empty
    return (...args) => { };
}
function wrapFunction(func, paramTypes, parameterProvider) {
    if (paramTypes == null || paramTypes.length == null || paramTypes.length === 0) {
        return func;
    }
    return ((...args) => {
        function resolveParameter(paramReflect, index) {
            return parameterProvider(args[index], paramReflect);
        }
        const functionParams = paramTypes.map(resolveParameter);
        return func(...functionParams);
    });
}
//# sourceMappingURL=constructor-injector.js.map